@@ -8,9 +8,9 @@
 
 # Package name      : perl5
 # Source directory  : .
-# Configuration time: Fri Sep 25 03:42:43 EDT 1998
-# Configured by     : gsar
-# Target system     : sunos 5.5.1 generic_103640-01 sun4m sparc sunw,sparcstation-20 
+# Configuration time: Sun Nov 29 18:17:40 EET 1998
+# Configured by     : jhi
+# Target system     : osf1 alpha.hut.fi v4.0 878 alpha 
 
 Author=''
 Date='$Date'
@@ -26,34 +26,35 @@ State=''
 _a='.a'
 _exe=''
 _o='.o'
-afs='true'
+afs='false'
 alignbytes='8'
 ansi2knr=''
 aphostname=''
-apiversion='5.00552'
+apiversion='5.00553'
 ar='ar'
-archlib='/l1/packages/gsperl/lib/5.00552/sun4-solaris-thread'
-archlibexp='/l1/packages/gsperl/lib/5.00552/sun4-solaris-thread'
-archname='sun4-solaris-thread'
+archlib='/opt/perl/lib/5.00553/alpha-dec_osf-thread'
+archlibexp='/opt/perl/lib/5.00553/alpha-dec_osf-thread'
+archname64=''
+archname='alpha-dec_osf-thread'
 archobjs=''
 awk='awk'
 baserev='5.0'
 bash=''
-bin='/l1/packages/gsperl/bin'
-binexp='/l1/packages/gsperl/bin'
+bin='/opt/perl/bin'
+binexp='/opt/perl/bin'
 bison=''
 byacc='byacc'
-byteorder='4321'
+byteorder='12345678'
 c='\c'
 castflags='0'
 cat='cat'
-cc='gcc'
-cccdlflags='-fPIC'
+cc='cc'
+cccdlflags=' '
 ccdlflags=' '
-ccflags='-D_REENTRANT -DDEBUGGING'
-cf_by='gsar'
+ccflags='-pthread -std -D__LANGUAGE_C__'
+cf_by='jhi'
 cf_email='yourname@yourhost.yourplace.com'
-cf_time='Fri Sep 25 03:42:43 EDT 1998'
+cf_time='Sun Nov 29 18:17:40 EET 1998'
 chgrp=''
 chmod=''
 chown=''
@@ -65,32 +66,33 @@ cp='cp'
 cpio=''
 cpp='cpp'
 cpp_stuff='42'
-cppflags='-D_REENTRANT -DDEBUGGING'
-cpplast='-'
-cppminus='-'
-cpprun='gcc -E'
-cppstdin='gcc -E'
+cppflags='-pthread -std -D__LANGUAGE_C__'
+cpplast=''
+cppminus=''
+cpprun='/usr/bin/cpp'
+cppstdin='cppstdin'
 cryptlib=''
 csh='csh'
-d_Gconvert='gconvert((x),(n),(t),(b))'
+d_Gconvert='gcvt((x),(n),(b))'
 d_access='define'
 d_accessx='undef'
 d_alarm='define'
 d_archlib='define'
-d_attribut='define'
+d_attribut='undef'
 d_bcmp='define'
 d_bcopy='define'
-d_bsd='define'
+d_bsd='undef'
 d_bsdgetpgrp='undef'
-d_bsdsetpgrp='undef'
+d_bsdsetpgrp='define'
 d_bzero='define'
-d_casti32='define'
+d_casti32='undef'
 d_castneg='define'
 d_charvspr='undef'
 d_chown='define'
 d_chroot='define'
 d_chsize='undef'
 d_closedir='define'
+d_cmsghdr_s='define'
 d_const='define'
 d_crypt='define'
 d_csh='define'
@@ -100,12 +102,13 @@ d_dbmclose64='undef'
 d_dbminit64='undef'
 d_delete64='undef'
 d_difftime='define'
-d_dirent64s='undef'
-d_dirnamlen='undef'
+d_dirent64_s='undef'
+d_dirnamlen='define'
 d_dlerror='define'
 d_dlopen='define'
 d_dlsymun='undef'
 d_dosuid='undef'
+d_drand48proto='define'
 d_dup2='define'
 d_eaccess='undef'
 d_endgrent='define'
@@ -127,8 +130,8 @@ d_fgetpos64='undef'
 d_fgetpos='define'
 d_firstkey64='undef'
 d_flexfnam='define'
-d_flock64s='undef'
-d_flock='undef'
+d_flock64_s='undef'
+d_flock='define'
 d_fopen64='undef'
 d_fork='define'
 d_fpathconf='define'
@@ -149,7 +152,7 @@ d_getgrps='define'
 d_gethbyaddr='define'
 d_gethbyname='define'
 d_gethent='define'
-d_gethname='undef'
+d_gethname='define'
 d_gethostprotos='define'
 d_getlogin='define'
 d_getnbyaddr='define'
@@ -175,9 +178,10 @@ d_gnulibc='undef'
 d_grpasswd='define'
 d_htonl='define'
 d_index='undef'
-d_inetaton='undef'
+d_inetaton='define'
 d_ino64t='undef'
 d_int64t='undef'
+d_iovec_s='define'
 d_isascii='define'
 d_killpg='define'
 d_lchown='define'
@@ -201,11 +205,17 @@ d_mkdir='define'
 d_mkfifo='define'
 d_mktime='define'
 d_msg='define'
+d_msg_ctrunc='define'
+d_msg_dontroute='define'
+d_msg_oob='define'
+d_msg_peek='define'
+d_msg_proxy='undef'
 d_msgctl='define'
 d_msgget='define'
+d_msghdr_s='define'
 d_msgrcv='define'
 d_msgsnd='define'
-d_mymalloc='define'
+d_mymalloc='undef'
 d_nextkey64='undef'
 d_nice='define'
 d_off64t='undef'
@@ -220,19 +230,21 @@ d_phostname='undef'
 d_pipe='define'
 d_poll='define'
 d_portable='define'
+d_pthread_create_joinable='undef'
 d_pthread_yield='undef'
-d_pthreads_created_joinable='define'
-d_pwage='define'
+d_pwage='undef'
 d_pwchange='undef'
 d_pwclass='undef'
 d_pwcomment='define'
 d_pwexpire='undef'
 d_pwgecos='define'
 d_pwpasswd='define'
-d_pwquota='undef'
+d_pwquota='define'
 d_readdir64='undef'
 d_readdir='define'
 d_readlink='define'
+d_readv='define'
+d_recvmsg='define'
 d_rename='define'
 d_rewinddir='define'
 d_rmdir='define'
@@ -240,15 +252,17 @@ d_safebcpy='define'
 d_safemcpy='undef'
 d_sanemcmp='define'
 d_sched_yield='define'
+d_scm_rights='define'
 d_seekdir64='undef'
 d_seekdir='define'
 d_select='define'
 d_sem='define'
 d_semctl='define'
-d_semctl_semid_ds='undef'
+d_semctl_semid_ds='define'
 d_semctl_semun='define'
 d_semget='define'
 d_semop='define'
+d_sendmsg='define'
 d_setegid='define'
 d_seteuid='define'
 d_setgrent='define'
@@ -267,8 +281,8 @@ d_setregid='define'
 d_setresgid='undef'
 d_setresuid='undef'
 d_setreuid='define'
-d_setrgid='undef'
-d_setruid='undef'
+d_setrgid='define'
+d_setruid='define'
 d_setsent='define'
 d_setsid='define'
 d_setvbuf='define'
@@ -299,7 +313,7 @@ d_strtod='define'
 d_strtol='define'
 d_strtoul='define'
 d_strxfrm='define'
-d_suidsafe='define'
+d_suidsafe='undef'
 d_symlink='define'
 d_syscall='define'
 d_sysconf='define'
@@ -317,7 +331,7 @@ d_truncate64='undef'
 d_truncate='define'
 d_tzname='define'
 d_umask='define'
-d_uname='define'
+d_uname='undef'
 d_union_semun='undef'
 d_vfork='undef'
 d_void_closedir='undef'
@@ -329,6 +343,7 @@ d_wait4='define'
 d_waitpid='define'
 d_wcstombs='define'
 d_wctomb='define'
+d_writev='define'
 d_xenix='undef'
 date='date'
 db_hashtype='u_int32_t'
@@ -338,7 +353,8 @@ direntrytype='struct dirent'
 dlext='so'
 dlsrc='dl_dlopen.xs'
 doublesize='8'
-dynamic_ext='B Data/Dumper Fcntl IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re'
+drand01='drand48()'
+dynamic_ext='B DB_File Data/Dumper Devel/Peek Fcntl IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re'
 eagain='EAGAIN'
 ebcdic='undef'
 echo='echo'
@@ -347,40 +363,41 @@ emacs=''
 eunicefix=':'
 exe_ext=''
 expr='expr'
-extensions='B Data/Dumper Fcntl IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re Errno'
-find='find'
+extensions='B DB_File Data/Dumper Devel/Peek Fcntl IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re Errno'
+find=''
 firstmakefile='makefile'
 flex=''
 fpostype='fpos_t'
 freetype='void'
 full_csh='/usr/bin/csh'
 full_sed='/usr/bin/sed'
-gccversion='2.8.1'
+gccversion=''
 gidtype='gid_t'
-glibpth='/usr/shlib /shlib /lib/pa1.1 /usr/lib/large /lib /usr/lib /usr/lib/386 /lib/386 /lib/large /usr/lib/small /lib/small /usr/ccs/lib /usr/local/lib'
+glibpth='/usr/shlib /usr/ccs/lib /usr/lib/cmplrs/cc /usr/lib /usr/local/lib /var/shlib'
 grep='grep'
 groupcat='cat /etc/group'
 groupstype='gid_t'
 gzip='gzip'
-h_fcntl='true'
-h_sysfile='false'
+h_fcntl='false'
+h_sysfile='true'
 hint='recommended'
 hostcat='cat /etc/hosts'
 huge=''
 i_arpainet='define'
 i_bsdioctl=''
-i_db='undef'
-i_dbm='undef'
+i_db='define'
+i_dbm='define'
 i_dirent='define'
 i_dld='undef'
 i_dlfcn='define'
-i_fcntl='define'
+i_fcntl='undef'
 i_float='define'
 i_gdbm='undef'
 i_grp='define'
 i_inttypes='undef'
 i_limits='define'
 i_locale='define'
+i_machcthr='undef'
 i_malloc='define'
 i_math='define'
 i_memory='undef'
@@ -388,24 +405,25 @@ i_ndbm='define'
 i_netdb='define'
 i_neterrno='undef'
 i_niin='define'
+i_poll='define'
 i_pwd='define'
-i_rpcsvcdbm='define'
+i_rpcsvcdbm='undef'
 i_sfio='undef'
 i_sgtty='undef'
 i_stdarg='define'
 i_stddef='define'
 i_stdlib='define'
 i_string='define'
-i_sysaccess='undef'
-i_sysdir='undef'
-i_sysfile='undef'
-i_sysfilio='define'
+i_sysaccess='define'
+i_sysdir='define'
+i_sysfile='define'
+i_sysfilio='undef'
 i_sysin='undef'
 i_sysioctl='define'
 i_sysndir='undef'
 i_sysparam='define'
 i_sysresrc='define'
-i_syssecrt='undef'
+i_syssecrt='define'
 i_sysselct='define'
 i_syssockio=''
 i_sysstat='define'
@@ -413,6 +431,7 @@ i_systime='define'
 i_systimek='undef'
 i_systimes='define'
 i_systypes='define'
+i_sysuio='define'
 i_sysun='define'
 i_syswait='define'
 i_termio='undef'
@@ -424,57 +443,58 @@ i_values='define'
 i_varargs='undef'
 i_varhdr='stdarg.h'
 i_vfork='undef'
+ignore_versioned_solibs=''
 incpath=''
 inews=''
-installarchlib='/l1/packages/gsperl/lib/5.00552/sun4-solaris-thread'
-installbin='/l1/packages/gsperl/bin'
-installman1dir='/l1/packages/gsperl/man/man1'
-installman3dir='/l1/packages/gsperl/man/man3'
-installprivlib='/l1/packages/gsperl/lib/5.00552'
-installscript='/l1/packages/gsperl/bin'
-installsitearch='/l1/packages/gsperl/lib/site_perl/5.00552/sun4-solaris-thread'
-installsitelib='/l1/packages/gsperl/lib/site_perl/5.00552'
+installarchlib='/opt/perl/lib/5.00553/alpha-dec_osf-thread'
+installbin='/opt/perl/bin'
+installman1dir='/opt/perl/man/man1'
+installman3dir='/opt/perl/man/man3'
+installprivlib='/opt/perl/lib/5.00553'
+installscript='/opt/perl/bin'
+installsitearch='/opt/perl/lib/site_perl/5.00553/alpha-dec_osf-thread'
+installsitelib='/opt/perl/lib/site_perl/5.00553'
 intsize='4'
-known_extensions='B DB_File Data/Dumper Fcntl GDBM_File IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re'
+known_extensions='B DB_File Data/Dumper Devel/Peek Fcntl GDBM_File IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re'
 ksh=''
 large=''
-ld='gcc'
-lddlflags='-G -L/usr/local/lib'
-ldflags=' -L/usr/local/lib'
+ld='ld'
+lddlflags='-shared -expect_unresolved "*" -O -msym -s'
+ldflags=''
 less='less'
 lib_ext='.a'
-libc=''
+libc='/usr/shlib/libc.so'
 libperl='libperl.a'
-libpth='/usr/local/lib /lib /usr/lib /usr/ccs/lib'
-libs='-lsocket -lnsl -ldl -lm -lposix4 -lpthread -lc'
-libswanted='sfio socket inet nsl nm ndbm gdbm dbm db dl dld sun m posix4 pthread c cposix posix ndir dir crypt bsd BSD PW x'
-line='line'
+libpth='/usr/shlib /usr/ccs/lib /usr/lib/cmplrs/cc /usr/lib /var/shlib'
+libs='-lgdbm -ldbm -ldb -lm -lpthread -lexc'
+libswanted='sfio socket inet nsl nm gdbm dbm db malloc dld ld sun m cposix posix ndir dir crypt ucb BSD x pthread exc'
+line=''
 lint=''
 lkflags=''
 ln='ln'
 lns='/usr/bin/ln -s'
 locincpth='/usr/local/include /opt/local/include /usr/gnu/include /opt/gnu/include /usr/GNU/include /opt/GNU/include'
 loclibpth='/usr/local/lib /opt/local/lib /usr/gnu/lib /opt/gnu/lib /usr/GNU/lib /opt/GNU/lib'
-longdblsize='16'
+longdblsize='8'
 longlongsize='8'
-longsize='4'
+longsize='8'
 lp=''
 lpr=''
 ls='ls'
-lseeksize='4'
+lseeksize='8'
 lseektype='off_t'
 mail=''
 mailx=''
 make='make'
 make_set_make='#'
-mallocobj='malloc.o'
-mallocsrc='malloc.c'
+mallocobj=''
+mallocsrc=''
 malloctype='void *'
-man1dir='/l1/packages/gsperl/man/man1'
-man1direxp='/l1/packages/gsperl/man/man1'
+man1dir='/opt/perl/man/man1'
+man1direxp='/opt/perl/man/man1'
 man1ext='1'
-man3dir='/l1/packages/gsperl/man/man3'
-man3direxp='/l1/packages/gsperl/man/man3'
+man3dir='/opt/perl/man/man3'
+man3direxp='/opt/perl/man/man3'
 man3ext='3'
 medium=''
 mips=''
@@ -484,15 +504,15 @@ models='none'
 modetype='mode_t'
 more='more'
 mv=''
-myarchname='sun4-solaris'
+myarchname='alpha-dec_osf'
 mydomain='.yourplace.com'
 myhostname='yourhost'
-myuname='sunos aatma 5.5.1 generic_103640-01 sun4m sparc sunw,sparcstation-20 '
+myuname='osf1 alpha.hut.fi v4.0 878 alpha '
 n=''
 netdb_hlen_type='int'
 netdb_host_type='const char *'
 netdb_name_type='const char *'
-netdb_net_type='long'
+netdb_net_type='int'
 nm='nm'
 nm_opt='-p'
 nm_so_opt=''
@@ -500,42 +520,47 @@ nonxs_ext='Errno'
 nroff='nroff'
 o_nonblock='O_NONBLOCK'
 obj_ext='.o'
-optimize='-g'
+optimize='-O'
 orderlib='false'
-osname='solaris'
-osvers='2.5.1'
+osname='dec_osf'
+osvers='4.0'
 package='perl5'
-pager='less'
+pager='/c/bin/less'
 passcat='cat /etc/passwd'
 patchlevel='5'
 path_sep=':'
-perl='perl'
+perl=''
 perladmin='yourname@yourhost.yourplace.com'
-perlpath='/l1/packages/gsperl/bin/perl'
+perlpath='/opt/perl/bin/perl'
 pg='pg'
-phostname='hostname'
+phostname=''
 pidtype='pid_t'
 plibpth=''
 pmake=''
 pr=''
-prefix='/l1/packages/gsperl'
-prefixexp='/l1/packages/gsperl'
-privlib='/l1/packages/gsperl/lib/5.00552'
-privlibexp='/l1/packages/gsperl/lib/5.00552'
+prefix='/opt/perl'
+prefixexp='/opt/perl'
+privlib='/opt/perl/lib/5.00553'
+privlibexp='/opt/perl/lib/5.00553'
 prototype='define'
-ptrsize='4'
-randbits='15'
+pthread_create_joinable='0'
+ptrsize='8'
+randbits='48'
+randfunc='drand48'
+randseedtype='long'
 ranlib=':'
 rd_nodata='-1'
 rm='rm'
 rmail=''
-runnm='false'
-scriptdir='/l1/packages/gsperl/bin'
-scriptdirexp='/l1/packages/gsperl/bin'
+runnm='true'
+sched_yield='sched_yield()'
+scriptdir='/opt/perl/bin'
+scriptdirexp='/opt/perl/bin'
 sed='sed'
+seedfunc='srand48'
 selectminbits='32'
 selecttype='fd_set *'
-sendmail='sendmail'
+sendmail=''
 sh='/bin/sh'
 shar=''
 sharpbang='#!'
@@ -543,14 +568,15 @@ shmattype='void *'
 shortsize='2'
 shrpenv=''
 shsharp='true'
-sig_name='ZERO HUP INT QUIT ILL TRAP ABRT EMT FPE KILL BUS SEGV SYS PIPE ALRM TERM USR1 USR2 CHLD PWR WINCH URG IO STOP TSTP CONT TTIN TTOU VTALRM PROF XCPU XFSZ WAITING LWP FREEZE THAW CANCEL RTMIN NUM38 NUM39 NUM40 NUM41 NUM42 NUM43 RTMAX IOT CLD POLL '
-sig_name_init='"ZERO", "HUP", "INT", "QUIT", "ILL", "TRAP", "ABRT", "EMT", "FPE", "KILL", "BUS", "SEGV", "SYS", "PIPE", "ALRM", "TERM", "USR1", "USR2", "CHLD", "PWR", "WINCH", "URG", "IO", "STOP", "TSTP", "CONT", "TTIN", "TTOU", "VTALRM", "PROF", "XCPU", "XFSZ", "WAITING", "LWP", "FREEZE", "THAW", "CANCEL", "RTMIN", "NUM38", "NUM39", "NUM40", "NUM41", "NUM42", "NUM43", "RTMAX", "IOT", "CLD", "POLL", 0'
-sig_num='0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 6, 18, 22, 0'
+sig_name='ZERO HUP INT QUIT ILL TRAP ABRT EMT FPE KILL BUS SEGV SYS PIPE ALRM TERM IOINT STOP TSTP CONT CHLD TTIN TTOU AIO XCPU XFSZ VTALRM PROF WINCH INFO USR1 USR2 RESV RTMIN NUM34 NUM35 NUM36 NUM37 NUM38 NUM39 NUM40 NUM41 NUM42 NUM43 NUM44 NUM45 NUM46 NUM47 MAX IOT LOST URG CLD IO POLL PTY PWR RTMAX '
+sig_name_init='"ZERO", "HUP", "INT", "QUIT", "ILL", "TRAP", "ABRT", "EMT", "FPE", "KILL", "BUS", "SEGV", "SYS", "PIPE", "ALRM", "TERM", "IOINT", "STOP", "TSTP", "CONT", "CHLD", "TTIN", "TTOU", "AIO", "XCPU", "XFSZ", "VTALRM", "PROF", "WINCH", "INFO", "USR1", "USR2", "RESV", "RTMIN", "NUM34", "NUM35", "NUM36", "NUM37", "NUM38", "NUM39", "NUM40", "NUM41", "NUM42", "NUM43", "NUM44", "NUM45", "NUM46", "NUM47", "MAX", "IOT", "LOST", "URG", "CLD", "IO", "POLL", "PTY", "PWR", "RTMAX", 0'
+sig_num='0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 6 6 16 20 23 23 23 29 48 '
+sig_num_init='0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 6, 6, 16, 20, 23, 23, 23, 29, 48, 0'
 signal_t='void'
-sitearch='/l1/packages/gsperl/lib/site_perl/5.00552/sun4-solaris-thread'
-sitearchexp='/l1/packages/gsperl/lib/site_perl/5.00552/sun4-solaris-thread'
-sitelib='/l1/packages/gsperl/lib/site_perl/5.00552'
-sitelibexp='/l1/packages/gsperl/lib/site_perl/5.00552'
+sitearch='/opt/perl/lib/site_perl/5.00553/alpha-dec_osf-thread'
+sitearchexp='/opt/perl/lib/site_perl/5.00553/alpha-dec_osf-thread'
+sitelib='/opt/perl/lib/site_perl/5.00553'
+sitelibexp='/opt/perl/lib/site_perl/5.00553'
 sizetype='size_t'
 sleep=''
 smail=''
@@ -564,7 +590,7 @@ spitshell='cat'
 split=''
 src='.'
 ssizetype='ssize_t'
-startperl='#!/l1/packages/gsperl/bin/perl'
+startperl='#!/opt/perl/bin/perl'
 startsh='#!/bin/sh'
 static_ext=' '
 stdchar='unsigned char'
@@ -575,7 +601,7 @@ stdio_filbuf=''
 stdio_ptr='((fp)->_ptr)'
 strings='/usr/include/string.h'
 submit=''
-subversion='52'
+subversion='53'
 sysman='/usr/man/man1'
 tail=''
 tar=''
@@ -591,9 +617,10 @@ troff=''
 uidtype='uid_t'
 uname='uname'
 uniq='uniq'
-use64bits='undef'
+use64bits='define'
 usedl='define'
-usemymalloc='y'
+usemultiplicity='undef'
+usemymalloc='n'
 usenm='true'
 useopcode='true'
 useperlio='undef'
@@ -604,7 +631,7 @@ usethreads='define'
 usevfork='false'
 usrinc='/usr/include'
 uuname=''
-version='5.00552'
+version='5.00553'
 vi=''
 voidflags='15'
 xlibpth='/usr/lib/386 /lib/386'
@@ -612,17 +639,20 @@ zcat=''
 zip='zip'
 # Configure command line arguments.
 config_arg0='Configure'
-config_args='-Dprefix=/opt/perl -Doptimize=-O -Dusethreads -Dcf_by=yourname -Dcf_email=yourname@yourhost.yourplace.com -Dperladmin=yourname@yourhost.yourplace.com -Dmydomain=.yourplace.com -Dmyhostname=yourhost -dE'
-config_argc=9
+config_args='-Dprefix=/opt/perl -Doptimize=-O -Dusethreads -Duse64bits -Dcf_by=yourname -Dcf_email=yourname@yourhost.yourplace.com -Dperladmin=yourname@yourhost.yourplace.com -Dmydomain=.yourplace.com -Dmyhostname=yourhost -dE'
+config_argc=10
 config_arg1='-Dprefix=/opt/perl'
 config_arg2='-Doptimize=-O'
 config_arg3='-Dusethreads'
-config_arg4='-Dcf_by=yourname'
-config_arg5='-Dcf_email=yourname@yourhost.yourplace.com'
-config_arg6='-Dperladmin=yourname@yourhost.yourplace.com'
-config_arg7='-Dmydomain=.yourplace.com'
-config_arg8='-Dmyhostname=yourhost'
-config_arg9='-dE'
+config_arg4='-Duse64bits'
+config_arg5='-Dcf_by=yourname'
+config_arg6='-Dcf_email=yourname@yourhost.yourplace.com'
+config_arg7='-Dperladmin=yourname@yourhost.yourplace.com'
+config_arg8='-Dmydomain=.yourplace.com'
+config_arg9='-Dmyhostname=yourhost'
+config_arg10='-dE'
 PATCHLEVEL=5
-SUBVERSION=52
+SUBVERSION=53
 CONFIG=true
+# Variables propagated from previous config.sh file.
+pp_sys_cflags='ccflags="$ccflags -DNO_EFF_ONLY_OK"'
