@@ -16,18 +16,18 @@ sub ext {
 
 sub _unix_os2_ext {
     my($self,$potential_libs, $verbose) = @_;
-    if ($^O =~ 'os2' and $Config{perllibs}) { 
+    if ($^O =~ 'os2' and $Config{libs}) { 
 	# Dynamic libraries are not transitive, so we may need including
 	# the libraries linked against perl.dll again.
 
 	$potential_libs .= " " if $potential_libs;
-	$potential_libs .= $Config{perllibs};
+	$potential_libs .= $Config{libs};
     }
     return ("", "", "", "") unless $potential_libs;
     warn "Potential libraries are '$potential_libs':\n" if $verbose;
 
     my($so)   = $Config{'so'};
-    my($libs) = $Config{'perllibs'};
+    my($libs) = $Config{'libs'};
     my $Config_libext = $Config{lib_ext} || ".a";
 
 
@@ -196,7 +196,7 @@ sub _win32_ext {
     my $BC		= 1 if $cc =~ /^bcc/i;
     my $GC		= 1 if $cc =~ /^gcc/i;
     my $so		= $Config{'so'};
-    my $libs		= $Config{'perllibs'};
+    my $libs		= $Config{'libs'};
     my $libpth		= $Config{'libpth'};
     my $libext		= $Config{'lib_ext'} || ".lib";
 
@@ -336,7 +336,7 @@ sub _vms_ext {
                  $self->{CCFLAS}   || $Config{'ccflags'};
   @crtls = ( ($dbgqual =~ m-/Debug-i ? $Config{'dbgprefix'} : '')
               . 'PerlShr/Share' );
-  push(@crtls, grep { not /\(/ } split /\s+/, $Config{'perllibs'});
+  push(@crtls, grep { not /\(/ } split /\s+/, $Config{'libs'});
   push(@crtls, grep { not /\(/ } split /\s+/, $Config{'libc'});
   # In general, we pass through the basic libraries from %Config unchanged.
   # The one exception is that if we're building in the Perl source tree, and
@@ -578,7 +578,7 @@ Unix-OS/2 version in several respects:
 =item *
 
 Input library and path specifications are accepted with or without the
-C<-l> and C<-L> prefixes used by Unix linkers.  If neither prefix is
+C<-l> and C<-L> prefices used by Unix linkers.  If neither prefix is
 present, a token is considered a directory to search if it is in fact
 a directory, and a library to search for otherwise.  Authors who wish
 their extensions to be portable to Unix or OS/2 should use the Unix
@@ -589,7 +589,7 @@ prefixes, since the Unix-OS/2 version of ext() requires them.
 Wherever possible, shareable images are preferred to object libraries,
 and object libraries to plain object files.  In accordance with VMS
 naming conventions, ext() looks for files named I<lib>shr and I<lib>rtl;
-it also looks for I<lib>lib and libI<lib> to accommodate Unix conventions
+it also looks for I<lib>lib and libI<lib> to accomodate Unix conventions
 used in some ported software.
 
 =item *
@@ -626,7 +626,7 @@ Unix-OS/2 version in several respects:
 =item *
 
 If C<$potential_libs> is empty, the return value will be empty.
-Otherwise, the libraries specified by C<$Config{perllibs}> (see Config.pm)
+Otherwise, the libraries specified by C<$Config{libs}> (see Config.pm)
 will be appended to the list of C<$potential_libs>.  The libraries
 will be searched for in the directories specified in C<$potential_libs>,
 C<$Config{libpth}>, and in C<$Config{installarchlib}/CORE>.
@@ -636,7 +636,7 @@ library pathnames is generated.
 =item *
 
 Input library and path specifications are accepted with or without the
-C<-l> and C<-L> prefixes used by Unix linkers.
+C<-l> and C<-L> prefices used by Unix linkers.
 
 An entry of the form C<-La:\foo> specifies the C<a:\foo> directory to look
 for the libraries that follow.
@@ -655,7 +655,7 @@ library to search for otherwise.  The C<$Config{lib_ext}> suffix will
 be appended to any entries that are not directories and don't already have
 the suffix.
 
-Note that the C<-L> and C<-l> prefixes are B<not required>, but authors
+Note that the C<-L> and <-l> prefixes are B<not required>, but authors
 who wish their extensions to be portable to Unix or OS/2 should use the
 prefixes, since the Unix-OS/2 version of ext() requires them.
 
@@ -670,7 +670,7 @@ Entries in C<$potential_libs> beginning with a colon and followed by
 alphanumeric characters are treated as flags.  Unknown flags will be ignored.
 
 An entry that matches C</:nodefault/i> disables the appending of default
-libraries found in C<$Config{perllibs}> (this should be only needed very rarely).
+libraries found in C<$Config{libs}> (this should be only needed very rarely).
 
 An entry that matches C</:nosearch/i> disables all searching for
 the libraries specified after it.  Translation of C<-Lfoo> and
@@ -680,7 +680,7 @@ valid files or directories.
 
 An entry that matches C</:search/i> reenables searching for
 the libraries specified after it.  You can put it at the end to
-enable searching for default libraries specified by C<$Config{perllibs}>.
+enable searching for default libraries specified by C<$Config{libs}>.
 
 =item *
 
