@@ -226,6 +226,7 @@ PP(pp_rv2gv)
     else {
 	if (SvTYPE(sv) != SVt_PVGV) {
 	    char *sym;
+	    STRLEN n_a;
 
 	    if (SvGMAGICAL(sv)) {
 		mg_get(sv);
@@ -240,7 +241,7 @@ PP(pp_rv2gv)
 		    warner(WARN_UNINITIALIZED, PL_warn_uninit);
 		RETSETUNDEF;
 	    }
-	    sym = SvPV(sv, PL_na);
+	    sym = SvPV(sv, n_a);
 	    if (PL_op->op_private & HINT_STRICT_REFS)
 		DIE(PL_no_symref, sym, "a symbol");
 	    sv = (SV*)gv_fetchpv(sym, TRUE, SVt_PVGV);
@@ -271,6 +272,7 @@ PP(pp_rv2sv)
     else {
 	GV *gv = (GV*)sv;
 	char *sym;
+	STRLEN n_a;
 
 	if (SvTYPE(gv) != SVt_PVGV) {
 	    if (SvGMAGICAL(sv)) {
@@ -286,7 +288,7 @@ PP(pp_rv2sv)
 		    warner(WARN_UNINITIALIZED, PL_warn_uninit);
 		RETSETUNDEF;
 	    }
-	    sym = SvPV(sv, PL_na);
+	    sym = SvPV(sv, n_a);
 	    if (PL_op->op_private & HINT_STRICT_REFS)
 		DIE(PL_no_symref, sym, "a SCALAR");
 	    gv = (GV*)gv_fetchpv(sym, TRUE, SVt_PV);
@@ -544,9 +546,10 @@ PP(pp_gelem)
     SV *tmpRef;
     char *elem;
     djSP;
-
+    STRLEN n_a;
+ 
     sv = POPs;
-    elem = SvPV(sv, PL_na);
+    elem = SvPV(sv, n_a);
     gv = (GV*)POPs;
     tmpRef = Nullsv;
     sv = Nullsv;
@@ -727,11 +730,11 @@ PP(pp_defined)
 	RETPUSHNO;
     switch (SvTYPE(sv)) {
     case SVt_PVAV:
-	if (AvMAX(sv) >= 0 || SvGMAGICAL(sv))
+	if (AvMAX(sv) >= 0 || SvGMAGICAL(sv) || (SvRMAGICAL(sv) && mg_find(sv,'P')))
 	    RETPUSHYES;
 	break;
     case SVt_PVHV:
-	if (HvARRAY(sv) || SvGMAGICAL(sv))
+	if (HvARRAY(sv) || SvGMAGICAL(sv) || (SvRMAGICAL(sv) && mg_find(sv,'P')))
 	    RETPUSHYES;
 	break;
     case SVt_PVCV:
@@ -762,8 +765,11 @@ PP(pp_undef)
 	RETPUSHUNDEF;
 
     if (SvTHINKFIRST(sv)) {
-	if (SvREADONLY(sv))
-	    RETPUSHUNDEF;
+	if (SvREADONLY(sv)) {
+	    dTHR;
+	    if (PL_curcop != &PL_compiling)
+		croak(PL_no_modify);
+	}
 	if (SvROK(sv))
 	    sv_unref(sv);
     }
@@ -1794,8 +1800,9 @@ PP(pp_hex)
     djSP; dTARGET;
     char *tmps;
     I32 argtype;
+    STRLEN n_a;
 
-    tmps = POPp;
+    tmps = POPpx;
     XPUSHu(scan_hex(tmps, 99, &argtype));
     RETURN;
 }
@@ -1806,8 +1813,9 @@ PP(pp_oct)
     UV value;
     I32 argtype;
     char *tmps;
+    STRLEN n_a;
 
-    tmps = POPp;
+    tmps = POPpx;
     while (*tmps && isSPACE(*tmps))
 	tmps++;
     if (*tmps == '0')
@@ -1919,7 +1927,8 @@ PP(pp_substr)
 	if (lvalue) {			/* it's an lvalue! */
 	    if (!SvGMAGICAL(sv)) {
 		if (SvROK(sv)) {
-		    SvPV_force(sv,PL_na);
+		    STRLEN n_a;
+		    SvPV_force(sv,n_a);
 		    if (ckWARN(WARN_SUBSTR))
 			warner(WARN_SUBSTR,
 				"Attempt to use reference as lvalue in substr");
@@ -2128,7 +2137,8 @@ PP(pp_ord)
 {
     djSP; dTARGET;
     UV value;
-    U8 *tmps = (U8*)POPp;
+    STRLEN n_a;
+    U8 *tmps = (U8*)POPpx;
     I32 retlen;
 
     if (IN_UTF8 && (*tmps & 0x80))
@@ -2171,12 +2181,13 @@ PP(pp_chr)
 PP(pp_crypt)
 {
     djSP; dTARGET; dPOPTOPssrl;
+    STRLEN n_a;
 #ifdef HAS_CRYPT
-    char *tmps = SvPV(left, PL_na);
+    char *tmps = SvPV(left, n_a);
 #ifdef FCRYPT
-    sv_setpv(TARG, fcrypt(tmps, SvPV(right, PL_na)));
+    sv_setpv(TARG, fcrypt(tmps, SvPV(right, n_a)));
 #else
-    sv_setpv(TARG, PerlProc_crypt(tmps, SvPV(right, PL_na)));
+    sv_setpv(TARG, PerlProc_crypt(tmps, SvPV(right, n_a)));
 #endif
 #else
     DIE(
@@ -2228,7 +2239,7 @@ PP(pp_ucfirst)
 	sv = TARG;
 	SETs(sv);
     }
-    s = (U8*)SvPV_force(sv, PL_na);
+    s = (U8*)SvPV_force(sv, slen);
     if (*s) {
 	if (PL_op->op_private & OPpLOCALE) {
 	    TAINT;
@@ -2284,7 +2295,7 @@ PP(pp_lcfirst)
 	sv = TARG;
 	SETs(sv);
     }
-    s = (U8*)SvPV_force(sv, PL_na);
+    s = (U8*)SvPV_force(sv, slen);
     if (*s) {
 	if (PL_op->op_private & OPpLOCALE) {
 	    TAINT;
@@ -2655,8 +2666,10 @@ PP(pp_hslice)
 		svp = avhv_fetch_ent((AV*)hv, keysv, lval, 0);
 	    }
 	    if (lval) {
-		if (!svp || *svp == &PL_sv_undef)
-		    DIE(PL_no_helem, SvPV(keysv, PL_na));
+		if (!svp || *svp == &PL_sv_undef) {
+		    STRLEN n_a;
+		    DIE(PL_no_helem, SvPV(keysv, n_a));
+		}
 		if (PL_op->op_private & OPpLVAL_INTRO)
 		    save_helem(hv, keysv, svp);
 	    }
@@ -3203,7 +3216,7 @@ PP(pp_unpack)
     unsigned int auint;
     U32 aulong;
 #ifdef HAS_QUAD
-    unsigned Quad_t auquad;
+    Uquad_t auquad;
 #endif
     char *aptr;
     float afloat;
@@ -3696,6 +3709,7 @@ PP(pp_unpack)
 		    }
 		    else if (++bytes >= sizeof(UV)) {	/* promote to string */
 			char *t;
+			STRLEN n_a;
 
 			sv = newSVpvf("%.*Vu", (int)TYPE_DIGITS(UV), auv);
 			while (s < strend) {
@@ -3705,7 +3719,7 @@ PP(pp_unpack)
 				break;
 			    }
 			}
-			t = SvPV(sv, PL_na);
+			t = SvPV(sv, n_a);
 			while (*t == '0')
 			    t++;
 			sv_chop(sv, t);
@@ -3760,11 +3774,11 @@ PP(pp_unpack)
 	    EXTEND(SP, len);
 	    EXTEND_MORTAL(len);
 	    while (len-- > 0) {
-		if (s + sizeof(unsigned Quad_t) > strend)
+		if (s + sizeof(Uquad_t) > strend)
 		    auquad = 0;
 		else {
-		    Copy(s, &auquad, 1, unsigned Quad_t);
-		    s += sizeof(unsigned Quad_t);
+		    Copy(s, &auquad, 1, Uquad_t);
+		    s += sizeof(Uquad_t);
 		}
 		sv = NEWSV(43, 0);
 		if (auquad <= UV_MAX)
@@ -3953,8 +3967,9 @@ doencodes(register SV *sv, register char *s, register I32 len)
 STATIC SV      *
 is_an_int(char *s, STRLEN l)
 {
+  STRLEN	 n_a;
   SV             *result = newSVpv("", l);
-  char           *result_c = SvPV(result, PL_na);	/* convenience */
+  char           *result_c = SvPV(result, n_a);	/* convenience */
   char           *out = result_c;
   bool            skip = 1;
   bool            ignore = 0;
@@ -4051,7 +4066,7 @@ PP(pp_pack)
     U32 aulong;
 #ifdef HAS_QUAD
     Quad_t aquad;
-    unsigned Quad_t auquad;
+    Uquad_t auquad;
 #endif
     char *aptr;
     float afloat;
@@ -4438,8 +4453,8 @@ PP(pp_pack)
 	case 'Q':
 	    while (len-- > 0) {
 		fromstr = NEXTFROM;
-		auquad = (unsigned Quad_t)SvIV(fromstr);
-		sv_catpvn(cat, (char*)&auquad, sizeof(unsigned Quad_t));
+		auquad = (Uquad_t)SvIV(fromstr);
+		sv_catpvn(cat, (char*)&auquad, sizeof(Uquad_t));
 	    }
 	    break;
 	case 'q':
@@ -4459,6 +4474,7 @@ PP(pp_pack)
 		if (fromstr == &PL_sv_undef)
 		    aptr = NULL;
 		else {
+		    STRLEN n_a;
 		    /* XXX better yet, could spirit away the string to
 		     * a safe spot and hang on to it until the result
 		     * of pack() (and all copies of the result) are
@@ -4468,9 +4484,9 @@ PP(pp_pack)
 			warner(WARN_UNSAFE,
 				"Attempt to pack pointer to temporary value");
 		    if (SvPOK(fromstr) || SvNIOK(fromstr))
-			aptr = SvPV(fromstr,PL_na);
+			aptr = SvPV(fromstr,n_a);
 		    else
-			aptr = SvPV_force(fromstr,PL_na);
+			aptr = SvPV_force(fromstr,n_a);
 		}
 		sv_catpvn(cat, (char*)&aptr, sizeof(char*));
 	    }
@@ -4669,7 +4685,7 @@ PP(pp_split)
     else {
 	maxiters += (strend - s) * rx->nparens;
 	while (s < strend && --limit &&
-	       CALLREGEXEC(rx, s, strend, orig, 1, Nullsv, NULL, 0))
+	       CALLREGEXEC(rx, s, strend, orig, 1, sv, NULL, 0))
 	{
 	    TAINT_IF(RX_MATCH_TAINTED(rx));
 	    if (rx->subbase
