@@ -430,22 +430,3 @@ __END__
 regkind=vars    # declared in regcomp.h
 simple=vars     # declared in regcomp.h
 varies=vars     # declared in regcomp.h
-watchaddr=vars  # declared in run.c
-watchok=vars    # declared in run.c
-yychar=vars     # generated by byacc in perly.c
-yycheck=vars    # generated by byacc in perly.c
-yydebug=vars    # generated by byacc in perly.c
-yydefred=vars   # generated by byacc in perly.c
-yydgoto=vars    # generated by byacc in perly.c
-yyerrflag=vars  # generated by byacc in perly.c
-yygindex=vars   # generated by byacc in perly.c
-yylen=vars      # generated by byacc in perly.c
-yylhs=vars      # generated by byacc in perly.c
-yylval=vars     # generated by byacc in perly.c
-yyname=vars     # generated by byacc in perly.c
-yynerrs=vars    # generated by byacc in perly.c
-yyrindex=vars   # generated by byacc in perly.c
-yyrule=vars     # generated by byacc in perly.c
-yysindex=vars   # generated by byacc in perly.c
-yytable=vars    # generated by byacc in perly.c
-yyval=vars      # generated by byacc in perly.c
