@@ -115,18 +115,14 @@ DllExport  long		win32_get_osfhandle(int fd);
 
 #ifndef USE_WIN32_RTL_ENV
 DllExport  char*	win32_getenv(const char *name);
-DllExport  int		win32_putenv(const char *name);
 #endif
 
 DllExport  unsigned 	win32_sleep(unsigned int);
 DllExport  int		win32_times(struct tms *timebuf);
 DllExport  unsigned 	win32_alarm(unsigned int sec);
 DllExport  int		win32_stat(const char *path, struct stat *buf);
-DllExport  char*	win32_longpath(char *path);
 DllExport  int		win32_ioctl(int i, unsigned int u, char *data);
-DllExport  int		win32_unlink(const char *f);
 DllExport  int		win32_utime(const char *f, struct utimbuf *t);
-DllExport  int		win32_uname(struct utsname *n);
 DllExport  int		win32_wait(int *status);
 DllExport  int		win32_waitpid(int pid, int *status, int flags);
 DllExport  int		win32_kill(int pid, int sig);
@@ -155,9 +151,7 @@ END_EXTERN_C
 #undef times
 #undef alarm
 #undef ioctl
-#undef unlink
 #undef utime
-#undef uname
 #undef wait
 
 #ifdef __BORLANDC__
@@ -210,7 +204,6 @@ END_EXTERN_C
 #define abort()			win32_abort()
 #define fstat(fd,bufptr)   	win32_fstat(fd,bufptr)
 #define stat(pth,bufptr)   	win32_stat(pth,bufptr)
-#define longpath(pth)   	win32_longpath(pth)
 #define rename(old,new)		win32_rename(old,new)
 #define setmode(fd,mode)	win32_setmode(fd,mode)
 #define lseek(fd,offset,orig)	win32_lseek(fd,offset,orig)
@@ -266,9 +259,7 @@ END_EXTERN_C
 #define times			win32_times
 #define alarm			win32_alarm
 #define ioctl			win32_ioctl
-#define unlink			win32_unlink
 #define utime			win32_utime
-#define uname			win32_uname
 #define wait			win32_wait
 #define waitpid			win32_waitpid
 #define kill			win32_kill
@@ -288,8 +279,6 @@ END_EXTERN_C
 #ifndef USE_WIN32_RTL_ENV
 #undef getenv
 #define getenv win32_getenv
-#undef putenv
-#define putenv win32_putenv
 #endif
 
 #endif /* WIN32IO_IS_STDIO */
